<?php
/**
 * GOMOOSTimeFilter Widget Class
 *
 * @project     CWC2
 * @revision    $Id:
 * @purpose     Filter layers by time
 * @author      DM Solutions Group (bronsema@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002-2003, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Label.php");

/**
 * GOMOOSTimeFilterWidget
 *
 * @desc A widget that displays a time range combobox to
 * display specific layers
 */
class GOMOOSTimeFilter extends CWCWidget
{
    /**
     * build a new widget
     */
    function GOMOOSTimeFilter()
    {
        $this->mszLanguageResource = str_replace("\\","/",dirname(__FILE__))."/GOMOOSTimeFilter.dbf";

        // invoke constructor of parent
        parent::CWCWidget();

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The GOMOOSTimeFilter widget allows the user to create a filter on the layers in
the current context, using a time value.
EOT;

        $this->mnPriority = PRIORITY_MEDIUM;
        $this->maAttributes["EARLIESTYEAR"] = new StringAttribute( "EARLIESTYEAR", false );
        $this->maAttributes["USECACHE"] = new BooleanAttribute( "USECACHE", false );
    }

    function InitDefaults()
    {
        return null;
    }

    function GetJavascriptOnLoadFunctions()
    {
        $aReturn = array();
        
        // validate current date on page load
        $szJsFunctionName = "monthChange";
        $szFunction = "$szJsFunctionName();\n";
        $aReturn[$szJsFunctionName] = $szFunction;
        
        return $aReturn;
    }

    /**
     * get the javascript functions for this widget
     */
    function GetJavascriptFunctions()
    {
        $aReturn = array();
        
        // define the leapyear function
        $szJsFunctionName = "leapYear";
        $szFunction = <<<EOT
/**
 * Check if a year is a leap year
 */
function {$szJsFunctionName}(year)
{
    if ((year/4)   != Math.floor(year/4))   return false;
    if ((year/100) != Math.floor(year/100)) return true;
    if ((year/400) != Math.floor(year/400)) return false;
    return true;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        // define the monthChange function
        $szJsFunctionName = 'monthChange';
        $szFunction = <<<EOT
/**
 * Update according to month change
 */
function {$szJsFunctionName}()
{
    // record current day
    var nDay = {$this->mszHTMLForm}.selDay.value;

    // re-build the list of days according to month
    {$this->mszHTMLForm}.selDay.options.length = 0;
    
    // get day count for current month and year
    var nDayCount = dayCount( {$this->mszHTMLForm}.selMonth.value,
                              {$this->mszHTMLForm}.selYear.value );
                              
    // loop and add values for days of the month
    for ( var i=1; i<=nDayCount; i++ )
    {
        {$this->mszHTMLForm}.selDay.options.length = i;
        {$this->mszHTMLForm}.selDay.options[i-1].value = i;
        {$this->mszHTMLForm}.selDay.options[i-1].text = i;
    }
    
    // reselect the previous day
    if ( nDay <= nDayCount )
        {$this->mszHTMLForm}.selDay.value = nDay;
    else
        {$this->mszHTMLForm}.selDay.value = nDayCount;
    
    // return
    return null;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        // define the dayCount function
        $szJsFunctionName = 'dayCount';
        $szFunction = <<<EOT
/**
 * Get day count according to month and year
 */
function {$szJsFunctionName}( month,year )
{
    // return day count for given month and year
    switch ( month )
    {
        case '1':
            return 31;
            break;
        case '2':
            if ( leapYear(year) )
                return 29;
            else
                return 28;
            break;
        case '3':
            return 31;
            break;
        case '4':
            return 30;
            break;
        case '5':
            return 31;
            break;
        case '6':
            return 30;
            break;
        case '7':
            return 31;
            break;
        case '8':
            return 31;
            break;
        case '9':
            return 30;
            break;
        case '10':
            return 31;
            break;
        case '11':
            return 30;
            break;
        case '12':
            return 31;
            break;
        default:
            return 31;
    }
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;
 

        return $aReturn;
    }

    /**
     * process the url looking for quickzoom requests
     */
    function  ParseURL()
    {
        parent::ParseURL();

        // get current settings
        $nDefaultHour = ( ( date('G') - 1 ) > 0 )? date('G') - 1 : 0 ;
        $nEarliestYear = $this->getValue("EarliestYear", "1940");
        $nCurrentYear  = (($this->isVarSet('selYear')) ?
                       $this->getVar('selYear') : date('Y') );
        $nCurrentMonth = (($this->isVarSet('selMonth')) ?
                       $this->getVar('selMonth') : date('m') );
        $nCurrentDay   = (($this->isVarSet('selDay')) ?
                       $this->getVar('selDay') : date('j') );
        $nCurrentHour  = (($this->isVarSet('selTime')) ?
                       $this->getVar('selTime') : $nDefaultHour );
        $nCurrentTimeZone = (($this->isVarSet('selTimeZone')) ?
                       $this->getVar('selTimeZone') : date('Z')/(60*60) );      
        $bUseCacheTime = $this->getValue("UseCache", "0");
        // get date/time as unix timestamp
        $nStartTime = mktime(( $nCurrentHour - $nCurrentTimeZone ),
                        0, 0, $nCurrentMonth , $nCurrentDay, $nCurrentYear );

        // convert time to formatted string
        $szStartTime = date("Y-m-j", $nStartTime)."%20".
          date("H:i:s", $nStartTime);
         
        // apply time filter to each layer
        $nLayers = $this->moMapObject->oMap->numlayers;
        for ($i=0;$i<$nLayers;$i++)
        {
            $oLayer = $this->moMapObject->oMap->getLayer($i);
            if ($oLayer->connectiontype == MS_WMS)
            {
                // check for the time parameter and remove it
                $szNewConnection = $this->cutTime( $oLayer->connection );
                
                // set the new connection and include time param
                $oLayer->set( 'connection', $szNewConnection.'&time='.$szStartTime );
                //$oLayer->setMetadata("wms_time", $szStartTime);
            }
            else
            {
                if($bUseCacheTime)
                {
                    $oLayer->set( 'filteritem', '_CWCTSTAMP' );
                    $oLayer->setFilter( $szStartTime );
                }
            }
        }

        // return
        return null;
    }
    
/**
 *  This function removes the time parameter from a connection string
 */    
    function cutTime( $szString )
    {
        // init vars
        $nStart = false;
        $nEnd = false;
        
        // check for a &time= or ?time=
        $nStart = strpos( $szString, '&time=' );
        if ( $nStart === false )
            $nStart = strpos( $szString, '?time=' );
        if ( $nStart === false )
            return $szString;
            
        // search for the next & after start
        $nEnd = strpos( $szString, '&', $nStart + 5 );
        if ( $nEnd === false )
            $nEnd = strlen( $szString );

        // cut the time parameter out and return
        return substr( $szString, 0, $nStart ).substr( $szString, $nEnd );
    }    

    /**
     * draw this widget
     */
    function DrawPublish()
    {
        // get current settings
        $nDefaultHour = ( ( date('G') - 1 ) > 0 )? date('G') - 1 : 0 ;
        $nEarliestYear = $this->getValue("EarliestYear", "1940");
        $nCurrentYear  = (($this->isVarSet('selYear')) ?
                       $this->getVar('selYear') : date('Y') );
        $nCurrentMonth = (($this->isVarSet('selMonth')) ?
                       $this->getVar('selMonth') : date('m') );
        $nCurrentDay   = (($this->isVarSet('selDay')) ?
                       $this->getVar('selDay') : date('j') );
        $nCurrentHour  = (($this->isVarSet('selTime')) ?
                       $this->getVar('selTime') : $nDefaultHour );
        $nCurrentTimeZone = (($this->isVarSet('selTimeZone')) ?
                       $this->getVar('selTimeZone') : date('Z')/(60*60) );      

        if (!$this->mbVisible)
            return "<!-- QuickView is hidden -->";

        // begin output
        $szResult  = '<!-- Start of GOMOOSTimeFilterWidget -->';
        $szResult .= '<table border="0" cellpadding="1" cellspacing="0">';
        $szResult .= '<tr>';
        $szResult .= '<td><span class="CWCTimeFilterLabelClass">'.
                            trim($this->moMLT->get('0','Date')).':</span></td>';
        
        // add month drop down
        $szResult .= '<td colspan="3"><select name="selMonth" onchange="monthChange()" class="CWCTimeFilterLabelClass">'."\n";
        for ( $i=1; $i<=12; $i++ )
        {
          if ( $nCurrentMonth == $i )
            $szResult .= '<option value="'.$i.'" selected>'.date('F', mktime (0,0,0,$i,1,1997)).'</option>'."\n";
          else
            $szResult .= '<option value="'.$i.'">'.date('F', mktime (0,0,0,$i,1,1997)).'</option>'."\n";
        }
        $szResult .= '</select>'."\n";
        
        // add day drop down
        $szResult .= '<span class="CWCTimeFilterLabelClass">&nbsp;</span><select name="selDay" class="CWCTimeFilterLabelClass">'."\n";
        for ( $i=1; $i<=31; $i++ )
        {
          if ($nCurrentDay == $i)
            $szResult .= '<option value="'.$i.'" selected>'.$i.'</option>'."\n";
          else
            $szResult .= '<option value="'.$i.'">'.$i.'</option>'."\n";
        }
        $szResult .= '</select>'."\n";

        // add year drop down
        $szResult .= '<span class="CWCTimeFilterLabelClass">&nbsp;</span><select name="selYear" onchange="monthChange()" class="CWCTimeFilterLabelClass">'."\n";
        for ($i=$nEarliestYear;  $i<=$nCurrentYear; $i++)
        {
          if ($nCurrentYear == $i)
            $szResult .= '<option value="'.$i.'" selected>'.$i.'</option>'."\n";
          else
            $szResult .= '<option value="'.$i.'">'.$i.'</option>'."\n";
        }
        $szResult .= '</select></td>';
        
        // close date row and start time row
        $szResult .= '</tr>'."\n";
        $szResult .= '<tr>'."\n";
        $szResult .= '<td><span class="CWCTimeFilterLabelClass">'.
                            trim($this->moMLT->get('1','Time')).':</span></td>';

        // add time drop down
        $szResult .= '<td><select name="selTime" class="CWCTimeFilterLabelClass">'."\n";
        for ( $i=0; $i<=23; $i++ )
        {
        if ($nCurrentHour == $i)
          $szResult .= '<option value="'.$i.'" selected>'.$i.':00:00</option>'."\n";
        else
          $szResult .= '<option value="'.$i.'">'.$i.':00:00</option>'."\n";
        }
        $szResult .= '</select></td>'."\n";
        
        // add time zone drop down
        $szResult .= '<td colspan="2" align="right"><span class="CWCTimeFilterLabelClass">'.
                    trim($this->moMLT->get('2','Time Zone')).':&nbsp;</span>';        
        $szResult .= '<select name="selTimeZone" class="CWCTimeFilterLabelClass">'."\n";
        for( $i=-12; $i<=12; $i++ )
        {
            // detemine sign
            $szSign = '+';
            if ( $i < 0 )
                $szSign = '-';
            elseif ( $i == 0 )
                $szSign = '&nbsp;&nbsp;';
            // pad with zeros
            $szTmpVal = $szSign.substr( trim( strval( abs( $i ) + 100 ) ), 1 );
            
            if ($nCurrentTimeZone == $i)
              $szResult .= '<option value="'.$i.'" selected>'.$szTmpVal.'</option>'."\n";
            else
              $szResult .= '<option value="'.$i.'">'.$szTmpVal.'</option>'."\n";
        }
        $szResult .= '</select></td>'."\n";
        
        // close table
        $szResult .= '</tr></table>'."\n";

        // return
        return $szResult;
    }

}
?>
